from PyQt5.QtWidgets import *
from PyQt5.QtCore import Qt
from joystick_simulation_node.controllers.components.generic_controller import GenericController
from joystick_simulation_node.controllers.components.joystick import Joystick
from joystick_simulation_node.controllers.components.slider_unclickable import SliderUnclickable
import math


class Extreme3DController(GenericController):
    def __init__(self, joystick_name : str, index : int) -> None:
        super(Extreme3DController, self).__init__(joystick_name, index, num_axes=4, num_buttons=12, num_povs=1)
        self.index = index
        self.initUI()

    def initUI(self):
        self.grid_layout = QGridLayout(self)
        self.setLayout(self.grid_layout)

        self.stick = Joystick()
        self.stick.valueChanged.connect(lambda value: self.set_joystick(value))

        slider_bar = SliderUnclickable(Qt.Horizontal)
        slider_bar.setFixedSize(150, 20)
        slider_bar.setMinimum(-20)
        slider_bar.setMaximum(20)
        slider_bar.valueChanged.connect(lambda value: self.set_twist(value))
        slider_bar.sliderReleased.connect(lambda button=slider_bar: self.release_slider(button))
    

        #row, column, total row span, total column span
        self.grid_layout.addWidget(self.stick,     0, 0)
        self.grid_layout.addWidget(slider_bar,     1, 0)


        button_pane = QWidget()
        button_pane_layout = QGridLayout()
        button_pane.setLayout(button_pane_layout)

        index = 0
        for j in range(0, math.ceil(self.num_buttons / 4) + 1):
            for i in range(0, 3):
                button = QPushButton(str(index))
                button.pressed.connect(lambda i=index: self.press_button(i))
                button.released.connect(lambda i=index: self.release_button(i))
                button_pane_layout.addWidget(button, j, i)
                index += 1
                if index >= self.num_buttons:
                    break

        self.grid_layout.addWidget(button_pane, 0, 1, 2, 1)

        self.adjustSize()

    def set_joystick(self, value):
        self.controller_data.axes[0] = value[0]
        self.controller_data.axes[1] = -value[1]

    def release_slider(self, button):
        button.setValue(0)

    def set_twist(self, value):
        self.controller_data.axes[2] = value / 20.0

    def press_button(self, i):
        self.controller_data.buttons[i] = 1

    def release_button(self, i):
        self.controller_data.buttons[i] = 0