from enum import Enum
from PyQt5.QtWidgets import QMainWindow, QWidget, QPushButton, QGridLayout, QRadioButton, QComboBox

class Alliance(Enum):
    RED = 0
    BLUE = 1
    
class State(Enum):
    DISABLED = 0
    AUTO = 1
    TELEOP = 2
    TEST = 3

class RobotStatusController(QWidget):
    def __init__(self) -> None:
        super(RobotStatusController, self).__init__()
        self.robot_enabled = False
        self.robot_alliance = Alliance.RED
        self.robot_state = State.DISABLED
        self.prev_robot_state = State.DISABLED

        self.initUI()


    def initUI(self):
        self.grid_layout = QGridLayout()
        self.setLayout(self.grid_layout)

        enable_disable_button = QPushButton("Enable")
        enable_disable_button.clicked.connect(lambda checked, button=enable_disable_button: self.toggle_enable_disable(button))

        radio_box = QWidget()
        radio_box_layout = QGridLayout()
        radio_box.setLayout(radio_box_layout)
        self.autonomous = QRadioButton("Autonomous")
        self.autonomous.setChecked(True)
        self.teleop = QRadioButton("Teleop")
        self.test_button = QRadioButton("Test")
        radio_box_layout.addWidget(self.autonomous, 0, 0)
        radio_box_layout.addWidget(self.teleop, 1, 0)
        radio_box_layout.addWidget(self.test_button, 2, 0)


        alliance_radio_box = QWidget()
        alliance_radio_box_layout = QGridLayout()
        alliance_radio_box.setLayout(alliance_radio_box_layout)
        red = QRadioButton("Red")
        red.setChecked(True)
        red.toggled.connect(lambda checked: self.set_robot_alliance(Alliance.RED))
        blue = QRadioButton("Blue")
        blue.toggled.connect(lambda checked: self.set_robot_alliance(Alliance.BLUE))
        alliance_radio_box_layout.addWidget(red, 0, 0)
        alliance_radio_box_layout.addWidget(blue, 1, 0)

        self.auto_selector = QComboBox()
        self.auto_selector.addItem('-- Use CyberSquire To Select Auto --')
        self.auto_selector.setEnabled(False)

        self.grid_layout.addWidget(enable_disable_button, 0, 0)
        self.grid_layout.addWidget(radio_box, 0, 1)
        self.grid_layout.addWidget(alliance_radio_box, 0, 2)
        self.grid_layout.addWidget(self.auto_selector, 1, 0, 1, 3)


    def toggle_enable_disable(self, button):
        self.robot_enabled = not self.robot_enabled
        if self.robot_enabled:
            if self.autonomous.isChecked():
                self.robot_state = State.AUTO
            elif self.teleop.isChecked():
                self.robot_state = State.TELEOP
            elif self.test_button.isChecked():
                self.robot_state = State.TEST
            button.setText("Disable")
            return
        else:
            self.robot_state = State.DISABLED
            button.setText("Enable")

    def set_robot_alliance(self, alliance):
        self.robot_alliance = alliance

    def set_auto_options(self, autos):
        self.auto_selector.setEnabled(True)
        self.auto_selector.clear()
        sorted_autos = autos.copy()
        sorted_autos.sort()
        self.auto_selector.addItems(sorted_autos)

    def get_selected_auto(self):
        return self.auto_selector.currentText()
