from PyQt5.QtWidgets import *
from PyQt5.QtCore import Qt
from joystick_simulation_node.controllers.components.generic_controller import GenericController
from joystick_simulation_node.controllers.components.joystick import Joystick
from joystick_simulation_node.controllers.components.slider_unclickable import SliderUnclickable

class RuffyController(GenericController):
    def __init__(self, joystick_name : str, index : int) -> None:
        super(RuffyController, self).__init__(joystick_name, index, num_axes=6, num_buttons=6, num_povs=0)
        self.index = index
        self.initUI()

    def initUI(self):
        self.grid_layout = QGridLayout(self)
        self.setLayout(self.grid_layout)

        self.left_stick = Joystick()
        self.left_stick.valueChanged.connect(lambda value: self.set_left_joystick(value))
        self.right_stick = Joystick()
        self.right_stick.valueChanged.connect(lambda value: self.set_right_joystick(value))

        left_slider_bar = SliderUnclickable(Qt.Horizontal)
        left_slider_bar.setFixedSize(150, 20)
        left_slider_bar.setMinimum(-20)
        left_slider_bar.setMaximum(20)
        left_slider_bar.valueChanged.connect(lambda value: self.set_left_twist(value))
        left_slider_bar.sliderReleased.connect(lambda button=left_slider_bar: self.release_slider(button))

        right_slider_bar = SliderUnclickable(Qt.Horizontal)
        right_slider_bar.setFixedSize(150, 20)
        right_slider_bar.setMinimum(-20)
        right_slider_bar.setMaximum(20)
        right_slider_bar.valueChanged.connect(lambda value: self.set_right_twist(value))
        right_slider_bar.sliderReleased.connect(lambda button=right_slider_bar: self.release_slider(button))
    

        #row, column, total row span, total column span
        self.grid_layout.addWidget(self.left_stick,     0, 0, 1, 2)
        self.grid_layout.addWidget(self.right_stick,    0, 2, 1, 2)
        self.grid_layout.addWidget(left_slider_bar,     1, 0, 1, 2)
        self.grid_layout.addWidget(right_slider_bar,    1, 2, 1, 2)

        for i in range(0, self.num_buttons):
            button = QPushButton(str(i))
            button.pressed.connect(lambda i=i: self.press_button(i))
            button.released.connect(lambda i=i: self.release_button(i))
            self.grid_layout.addWidget(button, 2, i)

        self.adjustSize()

    def set_left_joystick(self, value):
        self.controller_data.axes[0] = value[0]
        self.controller_data.axes[1] = value[1]

    def set_right_joystick(self, value):
        self.controller_data.axes[3] = value[0]
        self.controller_data.axes[4] = value[1]

    def release_slider(self, button):
        button.setValue(0)

    def set_left_twist(self, value):
        self.controller_data.axes[2] = value / 20.0

    def set_right_twist(self, value):
        self.controller_data.axes[5] = value / 20.0

    def press_button(self, i):
        self.controller_data.buttons[i] = 1

    def release_button(self, i):
        self.controller_data.buttons[i] = 0