from PyQt5 import QtWidgets, QtCore
from PyQt5.QtWidgets import *
from PyQt5.QtCore import Qt
from joystick_simulation_node.controllers.components.generic_controller import GenericController
from joystick_simulation_node.controllers.components.joystick import Joystick
from joystick_simulation_node.controllers.components.slider_unclickable import SliderUnclickable
from joystick_simulation_node.generated.ui_xbox_controller import Ui_XBoxController
import math
from ck_utilities_py_node.ckmath import normalize_to_2_pi

class XBoxController(GenericController):
    def __init__(self, joystick_name : str, index : int) -> None:
        super(XBoxController, self).__init__(joystick_name, index, num_axes=6, num_buttons=10, num_povs=1)
        self.setupUi()

    def setupUi(self):
        ui = Ui_XBoxController()
        ui.setupUi(self)
        self.setMinimumSize(503, 350)
        self.setMaximumSize(503, 350)

        left_stick = Joystick(self)
        left_stick.setGeometry(47, 21, 50, 50)
        left_stick.valueChanged.connect(lambda value: self.set_left_joystick(value))

        right_stick = Joystick(self)
        right_stick.setGeometry(224, 105, 50, 50)
        right_stick.valueChanged.connect(lambda value: self.set_right_joystick(value))

        pov_stick = Joystick(self)
        pov_stick.setGeometry(110, 105, 50, 50)
        pov_stick.valueChanged.connect(lambda value: self.set_pov(value))

        left_slider_bar = SliderUnclickable(self, orientation=Qt.Horizontal)
        left_slider_bar.setFixedSize(150, 20)
        left_slider_bar.move(10, 230)
        left_slider_bar.setMinimum(0)
        left_slider_bar.setMaximum(20)
        left_slider_bar.valueChanged.connect(lambda value: self.set_left_trigger(value))
        left_slider_bar.sliderReleased.connect(lambda button=left_slider_bar: self.release_slider(button))

        right_slider_bar = SliderUnclickable(self, orientation=Qt.Horizontal)
        right_slider_bar.setFixedSize(150, 20)
        right_slider_bar.move(340, 230)
        right_slider_bar.setMinimum(0)
        right_slider_bar.setMaximum(20)
        right_slider_bar.valueChanged.connect(lambda value: self.set_right_trigger(value))
        right_slider_bar.sliderReleased.connect(lambda button=right_slider_bar: self.release_slider(button))
    

        # #row, column, total row span, total column span
        # self.grid_layout.addWidget(self.left_stick,     0, 0, 1, 2)
        # self.grid_layout.addWidget(self.right_stick,    0, 2, 1, 2)
        # self.grid_layout.addWidget(left_slider_bar,     1, 0, 1, 2)
        # self.grid_layout.addWidget(right_slider_bar,    1, 2, 1, 2)

        # for i in range(0, self.num_buttons):
        #     button = QPushButton(str(i))
        #     button.pressed.connect(lambda i=i: self.press_button(i))
        #     button.released.connect(lambda i=i: self.press_button(i))
        #     self.grid_layout.addWidget(button, 2, i)

    def set_pov(self, value):
        #TODO: Fix - this does not work
        pov_value_set = [0, 45, 90, 135, 180, 225, 270, 315]
        if abs(value[0]) < 0.1 or abs(value[1]) < 0.1:
            self.controller_data.povs[0] = -1
        else:
            angle = math.degrees(normalize_to_2_pi(math.atan2(value[1], value[0])))
            self.controller_data.povs[0] = min(pov_value_set, key=lambda x:abs(x-angle))

        print(self.controller_data.povs[0])

    def set_left_joystick(self, value):
        self.controller_data.axes[0] = value[0]
        self.controller_data.axes[1] = value[1]

    def set_right_joystick(self, value):
        self.controller_data.axes[4] = value[0]
        self.controller_data.axes[5] = value[1]

    def release_slider(self, button):
        button.setValue(0)

    def set_left_trigger(self, value):
        self.controller_data.axes[2] = value / 20.0

    def set_right_trigger(self, value):
        self.controller_data.axes[3] = value / 20.0

    def press_button(self, i):
        self.controller_data.buttons[i] = 1

    def press_button(self, i):
        self.controller_data.buttons[i] = 0